/* 
 * This is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
 * USA.
 */
package android.androidVNC;

import java.io.File;
import java.io.PrintStream;
import java.sql.Date;

import android.os.Environment;
import android.util.Log;

/**
 * <p>(Se omiten acentos)</p>
 * <p>Clase para almacenar en formato HTML la informacion que suceda durante una ejecucion del programa.</p>
 * <p>Fichero: AndroidLog.java.</p>
 * <p>Aadido al PFC en la version v0.2.5.</p>
 * <p><b>Proyecto original:</b></p>
 * 		<p>Web: http://code.google.com/p/android-vnc-viewer/.</p>
 * 		<p>Release: 39.</p>
 *	 	<p>Migrada a release 60.</p>
 * @author Alberto Poblacion Navarro
 * 
 */
public class AndroidLog {
	/** Cadena para identifcar la clase en el log interno*/
	private static final String TAG = "AndroidLog";
	/** Flag para saber si esta marcado el checkbox del menu inicial*/
	private static boolean saveLog=true;
	/** Variable para almacenar las tramas enviadas */
	private int tramas_enviadas;
	/** Variable para almacenar las tramas recibidas correctamente */
	private int tramas_recibidas_correctas;
	/** Variable para almacenar las tramas recibidas erroneas y no identificadas */
	private int tramas_recibidas_erroneas;
	/** Objeto para escribir un flujo de datos */
	private PrintStream printStream = null;
	/** Objeto para trabajar con el archivo en el que vamos a escribir el registro */
	private File file;
	/** Objeto que almacena la unica instancia que debe crearse de esta clase en ejecucion. Patron Singleton */	
	private static AndroidLog instancia = null;
	/**
	 * Constructor publico de la clase. Solo se va a crear una instancia por ejecucion del programa.
	 * En caso de que no se haya creado la instancia se crea, en caso contrario se devuelve la creada anteriormente.
	 * @return Instancia de la clase.
	 */
	static public AndroidLog Instancia(){
		if (instancia == null)
			instancia = new AndroidLog();
		return instancia;
	}
	/** 
	 * Constructor privado de la clase. Se llamara solo una vez por ejecucin.
	 * Crea un objeto de la clase.
	 */
	private AndroidLog() {
		tramas_enviadas=0;
		tramas_recibidas_correctas=0;
		tramas_recibidas_erroneas=0;
		
		if(saveLog){	
			String path;
			String sdcard_status = Environment.getExternalStorageState();
			//Se comprueba si hay un medio extraible donde se pueda guardar el log
			if(sdcard_status.equals(Environment.MEDIA_MOUNTED)){
        		path = "/sdcard/"; //Si hay tarjeta se guarda en la tarjeta
			}else {
	        	//Si no hay tarjeta de memoria crearemos el log en la memoria interna del movil
	        	//Se crea en la carpeta del propio programa
	        	path = "/data/data/android.androidVNC/";
			}
			//Archivo que usara el buffer de salida para guardar la imagen
	        String nombre = new String("androidVNC.html");
			file = new File(path+nombre);
			
			try {
				file.createNewFile();
				//Mediante esta clase podemos escribir en el archivo directamente
				printStream = new PrintStream(file);
														
			} catch (Exception e) {
				Log.d(TAG, "Excepcion: "+e.toString());
				e.printStackTrace();
			}
			printStream.append("<html><body>"); //Cabezera del archivo HTML
			printStream.append("<title>AndroidVNC Log</title>");//Titulo
			printStream.append("<p><b>[INFO]<i> Ejecucion iniciada: "+new Date(System.currentTimeMillis()).toLocaleString()+"</i></b></p>");
			printStream.flush();//Se libera el buffer de escritura
			
		}
	}
	/**
	 * Aumentara en una unidad la cantidad de tramas enviadas del cliente al servidor
	 */
	public void envio_de_trama(){
		tramas_enviadas++;
	}
	/**
	 * Cuando se reciba una trama correcta se ha de llamar a esa funcion para 
	 * aumentar en una unidad el numero de tramas recibidas correctas
	 */
	public void recepcion_correcta(){
		tramas_recibidas_correctas++;
	}
	/**
	 * Cuando se reciba una trama erronea se ha de llamar a esa funcion para 
	 * aumentar en una unidad el numero de tramas recibidas erroneas
	 */
	public void recepcion_erronea(){
		tramas_recibidas_erroneas++;
	}
	/**
	 * Funcion que devuelve en forma de cadena las tramas enviadas / recibidas
	 * @return Cadena con la informacion de las tramas.
	 */
	public String toString(){
		return("<b>[Info]</b> Tramas Enviadas: " + tramas_enviadas
				+" Tramas Recibidas: Correctas:" +tramas_recibidas_correctas
				+" Erroneas: "+tramas_recibidas_erroneas+"\n");
	}
	/**
	 * Aade al registro en formato HTML la cadena pasada por parametro,
	 * como identificador de la clase se utilizara el parametro TAG 
	 * @param TAG Identificador de la clase donde se llame a la funcion.
	 * @param string Informacion que se va a imprimir en el log.
	 */
	public void log(String TAG, String string){
		if(saveLog){
		printStream.format("<p><b>[%s]</b> %s</p>",TAG, string);
		printStream.flush();
		}
	}
	/**
	 * Establece si se ha de guardar el registro segun el checkbox del menu inicial.
	 * @param save True si se ha de salvar el registro, false en caso contrario
	 * @return Devuelve el nuevo valor de la variable.
	 */
	static public boolean setsaveLog(boolean save){
		return saveLog = save;
	}
	/**
	 * Funcion para saber si se guardar o no el registro de eventos.
	 * @return True si se guarda el registro, false en caso contrario.
	 */
	static public boolean getsavelog() {
		return saveLog;
	}
	/**
	 * Cuando se llame a este procedimiento si se ha guardado el log:
	 * Se imprimen las estadisticas y se cierran las etiquetas HTML.
	 * Se libera y se cierra el buffer de escritura.
	 * Se libera la instancia para que se cree una nueva la proxima vez. 
	 */
	protected void finalizar(){
		if(saveLog){
			printStream.append("<p>"+toString()+"</p>");
			printStream.append("<p><b>[INFO]<i> Ejecucion finalizada: "+new Date(System.currentTimeMillis()).toLocaleString()+"</i></b></p>");
			printStream.append("</body></html>");
			printStream.flush();
			printStream.close();
		}
		instancia = null;	
	}
	/**
	 * Funcion para conocer si se ha creado un objeto de la clase.
	 * @return True si se ha creado una instancia, false en caso contrario.
	 */
	static boolean instanciado(){
		if (instancia == null)
			return false;
		else return true; 
	}	
		
}
