/* 
 * This is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
 * USA.
 */
/** 
	 * Copiar_y_Pegar.java
	 * Autor: Alberto Poblacin Navarro
	 * Aadido al proyecto en la version v0.2.2
	 * Aadido a:
	 		* Proyecto original: http://code.google.com/p/android-vnc-viewer/
	 		* Release: 39
	 		* Migrada a release 60.
	 */
package android.androidVNC;

import java.io.IOException;

import android.app.AlertDialog;
import android.content.Context;
import android.text.ClipboardManager;
import android.view.KeyEvent;
/**
 * <p>(Se omiten acentos)</p>
 * <p>Clase que permite compartir el portapapeles entre el cliente y el servidor VNC.</p> 
 * <p>Copiar_y_Pegar.java.</p>
 * <p>Aadido al PFC en la version v0.2.2.</p>
 * <p><b>Proyecto original:</b></p>
 * 		<p>Web: http://code.google.com/p/android-vnc-viewer/.</p>
 * 		<p>Release: 39.</p>
 *	 	<p>Migrada a release 60.</p>
 * @author Alberto Poblacion Navarro.
 */
public class Copiar_y_Pegar {
	/**
	 * Funcion que actualiza el clipboard remoto con la informacin del portapapeles local
	 * @param context Contexto de ejecucin del programa
	 * @param vncCanvas Lienzo que maneja los eventos de teclado
	 */
	static public void pegar(Context context){
		//Se obtiene el servicio de clipboard
		ClipboardManager portapapeles = (ClipboardManager)context.getSystemService("clipboard");
		VncCanvas vncCanvas = ((VncCanvasActivity)context).vncCanvas;
		try {
			//Se manda al servidor la informacin del portapapeles con la trama ClientCutText
			vncCanvas.rfb.writeClientCutText(portapapeles.getText().toString());
		} catch (IOException e) {
			//En caso de error se muestra cuadro de dilogo con la causa 
			new AlertDialog.Builder(context)
	          .setTitle(context.getString(R.string.no_paste))
	          .setMessage(context.getString(R.string.cause)+": "+e.toString())
	          .setIcon(android.R.drawable.ic_dialog_alert)
	          .setPositiveButton(android.R.string.ok, null)
	          .show();
			
		}
		//Se manda Control+V al remoto para que se pueda pegar automaticamente
		vncCanvas.processLocalKeyEvent(KeyEvent.KEYCODE_V,	new KeyEvent(0,0, KeyEvent.ACTION_DOWN, KeyEvent.KEYCODE_V, 0, KeyEvent.META_SYM_ON));
		//Se muestra en la barra de estado una notificacin
		Notificacion.Instancia(context).notificar(context.getString(R.string.paste),context.getString(R.string.clipboard_update));
	}
	/**
	 * Funcion que actualiza el clipboard local con la informacion del portapapeles remoto
	 * @param context Contexto de ejecucion del programa
	 * @param vncCanvas Lienzo que maneja los eventos de teclado.
	 * @param info Cadena con la informacion que se va a guardar en el portapapeles local.
	 */
	public static void copiar(Context context, String info) {
		//Se actualiza el clipboard local y se notifica al usuario.
		Notificacion.Instancia(context).notificar(context.getString(R.string.T_copy), context.getString(R.string.I_copy));
		ClipboardManager portapapeles = (ClipboardManager) context.getSystemService("clipboard");
		portapapeles.setText(info);
		
	}

}
