/* 
 * This is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
 * USA.
 */
package android.androidVNC;

import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import android.app.ListActivity;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.view.ContextMenu;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.ContextMenu.ContextMenuInfo;
import android.view.View.OnClickListener;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.ListView;
import android.widget.Toast;
import android.widget.AdapterView.OnItemLongClickListener;
/**
 * <p>(Se omiten acentos)</p>
 * <p>Clase que muestra por pantalla el arbol de ficheros de la tarjeta de memoria y permite elegir</p>
 *  un archivo para se enviado al servidor vnc.</p>
 * <p>Web de apoyo: http://groups.google.es/group/desarrolladores-android/web/construyendo-un-explorador-de-archivos?version=12.</p>
 * <p>Fichero: ExploradorArchivosLocal.java.</p>
 * <p>Aadido al PFC en la version v1.0.</p>
 * <p><b>Proyecto original:</b></p>
 * 		<p>Web: http://code.google.com/p/android-vnc-viewer/.</p>
 * 		<p>Release: 39.</p>
 *	 	<p>Migrada a release 60.</p>
 * @author Alberto Poblacion Navarro.
 */
public class ExploradorArchivosLocal extends ListActivity{
	/* Cadena para identifcar la clase en el log interno*/
    //private static final String TAG = "ExploradorArchivosLocal";
    /** Lista de cadenas con el path completo de los elementos en un directorio */
	private List<String> elementos_path = null; 
	/** Lista de cadenas con el nombre de los elementos en un directorio */
	private List<String> elementos_nombre = null;
	/** Variable donde se almacena el archivo que se va a manejar*/
    private File archivo;
    /** Vista para mostrar los archivos en un directorio*/
    private ListView listView;
    /** Boton para confirmar el envio de un archivo al servidor */
    private Button boton_enviar;
    /** Boton para cancelar la eleccion de un archvio */
    private Button boton_cancelar;
    /** Lugar de la lista seleccionado */
    int posicion;
    
    /**
     * Funcion llamada automaticamente cuando se crea esta actividad.
     * Se muestra el directorio donde se monta la tarjeta de memoria "/sdcard".
     * @author Alberto Poblacion Navarro. 
     */
    public void onCreate(Bundle icicle) {
	        super.onCreate(icicle);
	        setContentView(R.layout.listado);
	        rellenarConElRaiz();
	        setTitle(getString(R.string.select_file));
	        
	        //Se obtiene la lista y se configura para elegir solo un archivo simultaneamente
	        listView = getListView();
	        listView.setChoiceMode(ListView.CHOICE_MODE_SINGLE);
	 
	        final Context contex = this;
	        //Se configura el comportamiento de los botones
	        boton_enviar = (Button) findViewById(R.id.boton1);
	        boton_enviar.setText(getString(R.string.button_send));
	        boton_enviar.setEnabled(false);
	        boton_enviar.setOnClickListener(new OnClickListener(){
				@Override
				public void onClick(View v) {
					//Se lanza una nueva actividad para elegir donde se copiara el archivo
					Intent listaremoto = new Intent (contex, ExploradorArchivosRemoto.class);
                	listaremoto.putExtra(ExploradorArchivosRemoto.TIPO, ExploradorArchivosRemoto.ENVIAR);
 					startActivityForResult(listaremoto, 0);
				}
	        	
	        });
	        boton_cancelar = (Button) findViewById(R.id.boton2);
	        boton_cancelar.setText(getString(R.string.button_cancel));
	        boton_cancelar.setOnClickListener(new OnClickListener(){
				@Override
				public void onClick(View v) {
					//Se cierra la actividad sin elegir ningun archivo
					setResult(RESULT_CANCELED);
					finish();
				}
	        });
			listView.setOnItemLongClickListener(new OnItemLongClickListener(){
				@Override
				public boolean onItemLongClick(AdapterView<?> arg0, View arg1,
						int arg2, long arg3) {
					posicion=arg2;
					registerForContextMenu(arg0);
					return false;
				}
	        	
	        });

	}
	/**
	 * Funcion que se llamara cuando se destruya la actividad.
	 * Si no se ha elegido ningun archivo se noticara como tal a la actividad que recoja su codigo de retorno
	 */
	public void onDestroy() {
    	super.onDestroy();
    	setResult(RESULT_CANCELED, null);
    
   	}
	/**
	 * Muestra el listado de ficheros desde el directorio de la tarjeta de memoria "/sdcard".
	 */
	private void rellenarConElRaiz() {
	        rellenar(new File("/sdcard").listFiles());
	}
	/**
	 * Se invocara esta funcion cuando se seleccione una opcion de la ListView
	 * Si se selecciona un directorio se muestra el contenido.
	 * Si se hace clic en en fichero se selecciona.
	 * @param l Vista de la lista
	 * @param v Vista asiciada a la vista
	 * @param position Posicion seleccionada
	 * @para id Identificador del objeto seleccionado
	 */
	@Override
    protected void onListItemClick(ListView l, View v, int position, long id) {
        int IDFilaSeleccionada = position;
        if (IDFilaSeleccionada==0){
            rellenarConElRaiz();
        } else {
        	//Se evita una serie de caracteres que no son compatibles en la codificacion de android y windows
        	Pattern p = Pattern.compile("[a-zA-Z0-9._\\s\\p{Punct}-()&&[^]]*");
			Matcher m = p.matcher(l.getItemAtPosition(position).toString());
			if(m.matches()){
	        	boton_enviar.setEnabled(false);
	            archivo = new File(elementos_path.get(IDFilaSeleccionada));
	            //Si el archivo seleccionado es un directorio se muestra su contenido
	            if (archivo.isDirectory())
	                rellenar(archivo.listFiles());
	             else{
	            	boton_enviar.setEnabled(true);
	            	//Si es un fichero se selecciona en la lista.
	            	listView.setItemChecked(position, true);
	            	l.setItemChecked(position, true);
	             	}
			}else{
				Context context = this;
				Toast.makeText(context, getString(R.string.codification_problem), Toast.LENGTH_SHORT).show();
				boton_enviar.setEnabled(false);
			}
        }
    }
	/**
	 * Muestra por pantalla la lista de ficheros contenida en el array pasado por parametro.
	 * @param archivos Vector de ficheros.
	 */
	private void rellenar(File[] archivos) {
        elementos_path = new ArrayList<String>();
        elementos_path.add("[Volver Arriba]");
        elementos_nombre = new ArrayList<String>();
        elementos_nombre.add("[Volver Arriba]");
        for( File archivo: archivos){
            elementos_path.add(archivo.getPath());
            elementos_nombre.add(archivo.getName());
        }
        ArrayAdapter<String> listaArchivos= new ArrayAdapter<String>(this, android.R.layout.simple_list_item_single_choice, elementos_nombre);
        setListAdapter(listaArchivos); 
    }  
	/** 
	 * Se llama automaticamente esta funcion cuando finaliza lanzada en esta clase: ExploradorArchivosRemoto.java
	 * Si el resultado es positivo se manda el archivo a enviar y el path de destino a la clase vncCanvasActivity.
	 * Si el resultado es negativo se finaliza la clase.
	 * @param requestCode Codigo de peticion
	 * @param resultCode Codigo de retorno
	 * @intent data Informacion adicional
	 */
	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {

		if(resultCode == RESULT_OK){
			//Se aade al intent, la informacion del path remoto y el archivo a enviar.
			data.putExtra("path",data.getStringExtra("path"));
			data.putExtra("archivo", archivo.getAbsolutePath());
       	 	setResult(RESULT_OK, data);
		} else setResult(RESULT_CANCELED, data); 
       	  	finish();
	}
	
	/**
	 * Funcion que se llama cuando se lanza un menu conextual.
	 * Se llamara cuando se deje pulsada una opcion de la ListView durante un tiempo prolongado. 
	 * Dependiendo de si se selecciona una carpeta o un archivo mostrara una opcion u otra.
	 * @param menu Contexto del menu
	 * @param v Vista asociada al menu
	 * @param menuInfo Informacion del menu
	 */
    @Override
    public void onCreateContextMenu(ContextMenu menu, View v,
    		ContextMenuInfo menuInfo) {

    	super.onCreateContextMenu(menu, v, menuInfo);
    	menu.clear();
    	menu.setHeaderTitle("Opciones");
    	if (new File(elementos_path.get(posicion)).isDirectory())
    		menu.add(0,2,Menu.NONE,"Abrir");
    	else{ 
    		menu.add(0,1,Menu.NONE,"Seleccionar");
    		menu.add(0,0,Menu.NONE,"Eliminar");
    	}
    	
    }
    /**
     * Funcion llamada automaticamente cuando se seleccione una opcion del menu contextual.
     * Permite liminar un archivo, seleccionarlo o abrir un directorio.
     * @param item Opcion seleccionada
     */
    @Override
    public boolean onContextItemSelected(MenuItem item) {
    	File archivo =new File(elementos_path.get(posicion));
    	Pattern p = Pattern.compile("[a-zA-Z0-9._\\s\\p{Punct}-()&&[^]]*");
		Matcher m = p.matcher(archivo.getName());
		if(m.matches()){
	    	switch (item.getItemId()) {
			case 0:
				archivo.delete();
				rellenar(archivo.getParentFile().listFiles());
				break;
			case 1:
				listView.setItemChecked(posicion, true);
				boton_enviar.setEnabled(true);
				break;
			case 2:
				rellenar(archivo.listFiles());
				break;
	    
	    	}
		}else Toast.makeText(this, getString(R.string.codification_problem), Toast.LENGTH_SHORT).show();
    	return super.onContextItemSelected(item);
    }
	
}

	