/* 
 * This is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
 * USA.
 */
package android.androidVNC;

import java.util.List;

import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.DialogInterface.OnClickListener;
import android.content.pm.ActivityInfo;
import android.hardware.Sensor;
import android.hardware.SensorEvent;
import android.hardware.SensorEventListener;
import android.hardware.SensorManager;
import android.util.Log;
import android.view.Display;

/**
 * <p>(Se omiten acentos) </p>
 * <p>Clase encargada de realizar panning en el escritorio remoto mediante el uso de los sensores
 * internos del telefono.</p>
 * <p>Sensores.java.</p>
 * <p>Aadido al PFC en la version v0.1.2</p>
 * <p><b>Proyecto original:</b></p>
 * 		<p>Web: http://code.google.com/p/android-vnc-viewer/.</p>
 * 		<p>Release: 39.</p>
 *	 	<p>Migrada a release 60.</p>
 * @author Alberto Poblacion Navarro.
 */
public class Sensores{
	/** Cadena para identificar la clase en el log interno*/
	private static final String TAG = "SENSORES";
	/** Objeto encargado de manejar los sensores*/
	private SensorManager sensorManager;
	/** Objeto que representa un sensor concreto */
	private Sensor sensor;
	/** Oyente que obtiene la informacion procedente de los sensores */
	private SensorEventListener sensorEventListener;
	/** Lista de sensores donde se guardaran aquellos que esten disponibles */
	private List<Sensor> lista_sensores;
	/** Instancia de la clase encargada de manejar los eventos de entrada */
	private VncCanvasActivity vncCanvasActivity;
	/** Instancia de la clase encargada de dibujar la imagen */
	private VncCanvas vncCanvas;
	/** Clase que permite notificar los sucesos que se produzcan durante la ejecucion */
	private Notificacion notificacion;
	/** Clase para escribir en el registro HTML los sucesos importantes en la ejecucion */
	private AndroidLog log;
	/** Flag para registrar la posicion actual en los distintos ejes */
	private boolean registrar_posicion;
	/** Valor inicial en los distintos ejes */
	private float ejex, ejey, ejez;
	/** Cantidad de pixeles que se desplazan por el escritorio */
	private int MOVER=30; 
	/** Posicion en la que se encuentra el acelerometro en la lista de sensores*/
	static final int ACELEROMETRO = 0;
	/** Posicion en la que se encuentra el sensor de orientacion en la lista de sensores*/
	static final int ORIENTACION = 1;
	/** Posicion en la que se encuentra el sensor de campo magnetico en la lista de sensores*/
	static final int BRUJULA = 2;
	/** Objeto que almacena la unica instancia que debe crearse de esta clase en ejecucion. Patron Singleton */	
	private static Sensores instancia = null;
	/**
	 * Constructor publico de la clase. Solo se va a crear una instancia por ejecucion del programa.
	 * En caso de que no se haya creado la instacia se crea, en caso contrario se devuelve la creada anteriormente.
	 * @return Instancia de la clase.
	 */
	static synchronized Sensores Instancia(VncCanvasActivity vnccanvasactivity, VncCanvas vnccanvas){
		if (instancia == null)
			instancia = new Sensores(vnccanvasactivity, vnccanvas);
		return instancia;
	}
	/**
	 * Crea una instancia de la clase sensores, obteniendo los sensores existentes en el dispositivo.
	 * Inicializa las variables.
	 * @param vnccanvasactivity Instancia de la clase encargada de manejar los eventos de entrada 
	 * @param vnccanvas Instancia de la clase encargada de dibujar la imagen
	 */ 
	private Sensores(final VncCanvasActivity vnccanvasactivity, VncCanvas vnccanvas){
		
			vncCanvas = vnccanvas;
			vncCanvasActivity = vnccanvasactivity;
			
			registrar_posicion=true;
			//Se obienen las instancias de los distintos recursos.
			notificacion = Notificacion.Instancia(vncCanvasActivity);
			log = AndroidLog.Instancia();
			//Se obtiene un manejador de sensores, la lista de sensores disponibles y un oyente para registrar los valores de los sensores
			sensorManager=(SensorManager)vnccanvasactivity.getSystemService(Context.SENSOR_SERVICE);
			lista_sensores = sensorManager.getSensorList(Sensor.TYPE_ALL);
			sensorEventListener = new SensorEventListener(){
      	    	@Override
				public void onAccuracyChanged(Sensor sensor, int accuracy) {
			    		//No hacer nada					
				}
				@Override
				public void onSensorChanged(SensorEvent event) {
					int limite = 10;
					if(event.sensor.getType() == Sensor.TYPE_ORIENTATION){
						//Si los cambios se producen en el sensor orientacion
						int scrollX=0;
						int scrollY=0;
						
						if(registrar_posicion) //Se guarda la referencia de la posicion inicial
						{
							ejex=event.values[SensorManager.DATA_X];
							ejey=event.values[SensorManager.DATA_Y];
							ejez=event.values[SensorManager.DATA_Z];
							registrar_posicion = false;
						}
						if(event.values[SensorManager.DATA_Y] > (ejey+limite)){ //Extremo izquierdo hacia atras	
							//Desplazar escritorio hacia la izquierda
							scrollX -= MOVER;
						}else if (event.values[SensorManager.DATA_Y]+limite < ejey){ //Extremo derecho hacia atras 
							//Desplazar escritorio hacia la derecha
							scrollX +=MOVER;
						}if(event.values[SensorManager.DATA_Z] > (ejez+limite)){	//Pantalla hacia arriba
							//Desplazar escritorio hacia abajo
							scrollY += MOVER;
						}else if (event.values[SensorManager.DATA_Z]+limite < ejez){ //Pantalla hacia abajo 
							//Desplazar escritorio hacia arriba
							scrollY -= MOVER;
						}	
						//Cantidad de desplazamiento a realizar
						vncCanvasActivity.absoluteXPosition += scrollX;
						vncCanvasActivity.absoluteYPosition += scrollY;
						
						Display display=vncCanvasActivity.getWindowManager().getDefaultDisplay();
						//Se comprueban los margenes superior e izquierdo
						if(vncCanvasActivity.absoluteXPosition < 0){
							scrollX=0;
							vncCanvasActivity.absoluteXPosition=0;
							vncCanvas.scrollTo(vncCanvasActivity.absoluteXPosition, vncCanvas.getScrollY());
						}
						if(vncCanvasActivity.absoluteYPosition < 0){
							scrollY=0;
							vncCanvasActivity.absoluteYPosition=0;
							vncCanvas.scrollTo(vncCanvas.getScrollX(), vncCanvasActivity.absoluteYPosition);
						}
						//Se comprueban los margenes derecho y inferior
						if(vncCanvasActivity.absoluteXPosition+ display.getWidth()> vncCanvas.getImageWidth()){
							vncCanvasActivity.absoluteXPosition=vncCanvas.getImageWidth()-(display.getWidth());
							vncCanvas.scrollTo(0, vncCanvas.getScrollY());
							vncCanvas.scrollTo(vncCanvasActivity.absoluteXPosition,vncCanvas.getScrollY());
							scrollX=0;
						}
						if(vncCanvasActivity.absoluteYPosition + display.getHeight()> vncCanvas.getImageHeight()){
							vncCanvasActivity.absoluteYPosition=vncCanvas.getImageHeight()-(vncCanvas.getHeight());
							vncCanvas.scrollTo(vncCanvas.getScrollX(),0);
							vncCanvas.scrollTo(vncCanvas.getScrollX(), vncCanvasActivity.absoluteYPosition);
							scrollY=0;
						}
						vncCanvas.scrollBy(scrollX, scrollY);
						
					}if(event.sensor.getType() == Sensor.TYPE_MAGNETIC_FIELD){
						//Si los cambios se producen en el sensor de campo magnetico 
						MOVER = 50;
						if(registrar_posicion)
						{
							ejex=event.values[SensorManager.DATA_X];
							ejey=event.values[SensorManager.DATA_Y];
							ejez=event.values[SensorManager.DATA_Z];
							registrar_posicion = false;
							
						}
						int scrollX=0;
						int scrollY=0;
						float margen = 1;
						//Desplazamiento del escritorio lateral
						if(event.values[SensorManager.DATA_X] > ejex+margen){
							//Mover Escritorio a la izquierda
							ejex =event.values[SensorManager.DATA_X];
							scrollX -=MOVER;
							
							
						}else if(event.values[SensorManager.DATA_X]+margen < ejex){
							//Mover Escritorio a la derecha
							ejex =event.values[SensorManager.DATA_X];
							scrollX +=MOVER;
							
						}
						//Desplazamiento del escritorio vertical
						if(event.values[SensorManager.DATA_Y] > ejey+margen){
							//Mover Escritorio a abajo
							ejey =event.values[SensorManager.DATA_Y];
							scrollY += MOVER;
							
						}else if(event.values[SensorManager.DATA_Y]+margen< ejey){
							//Mover Escritorio a arriba
							ejey =event.values[SensorManager.DATA_Y];
							scrollY -= MOVER;
						}
						//Cantidad de desplazamiento a realizar
						vncCanvasActivity.absoluteXPosition += scrollX;
						vncCanvasActivity.absoluteYPosition += scrollY;
						
						Display display=vncCanvasActivity.getWindowManager().getDefaultDisplay();
						//Se comprueban los margenes superior e izquierdo
						if(vncCanvasActivity.absoluteXPosition < 0){
							scrollX=0;
							vncCanvasActivity.absoluteXPosition=0;
							vncCanvas.scrollTo(vncCanvasActivity.absoluteXPosition, vncCanvas.getScrollY());
						}
						if(vncCanvasActivity.absoluteYPosition < 0){
							scrollY=0;
							vncCanvasActivity.absoluteYPosition=0;
							vncCanvas.scrollTo(vncCanvas.getScrollX(), vncCanvasActivity.absoluteYPosition);
						}
						//Se comprueban los margenes derecho e inferior
						if(vncCanvasActivity.absoluteXPosition+ display.getWidth()> vncCanvas.getImageWidth()){
							vncCanvasActivity.absoluteXPosition=vncCanvas.getImageWidth()-(display.getWidth());
							vncCanvas.scrollTo(0, vncCanvas.getScrollY());
							vncCanvas.scrollTo(vncCanvasActivity.absoluteXPosition,vncCanvas.getScrollY());
							scrollX=0;
						}
						if(vncCanvasActivity.absoluteYPosition + display.getHeight()> vncCanvas.getImageHeight()){
							vncCanvasActivity.absoluteYPosition=vncCanvas.getImageHeight()-(vncCanvas.getHeight());
							vncCanvas.scrollTo(vncCanvas.getScrollX(),0);
							vncCanvas.scrollTo(vncCanvas.getScrollX(), vncCanvasActivity.absoluteYPosition);
							scrollY=0;
						}
						vncCanvas.scrollBy(scrollX, scrollY);
					}
						
						
					}
										
					
				};
				
	        };
	        
	/**
	 * Registra los sensores en el manejador y se comportan segun el comportamiento descrito en el oyente.
	 * Mediante el sensor de campo magnetico se realiza un movimiento de la posicion del escritorio visible desplazando el telefono.
	 * Mediante el sensor de orientacion se puede realizar un movimiento de la posicion del escritorio variando la orientacion del movil. 
	 * @return Devuelve true si se consigue registrar con exito un sensor, false en caso contrario
	 * @author Alberto Poblacion Navarro.
	 */
	 public boolean registrar(int tipo){
		 boolean encontrado = false;
	 		//Se muestra un cuadro de dialogo para que se coloque el telefono en la posicion adecuada y se registra el sensor cuando se acepte el mensaje
			switch (tipo) {
			case ORIENTACION:
				sensorManager.unregisterListener(sensorEventListener);
				for (int i=0; i<lista_sensores.size();i++){
					sensor = lista_sensores.get(i);
					if(sensor.getType()==Sensor.TYPE_ORIENTATION){
						encontrado=true;
						break;
						}
				}
				if(encontrado){
					new AlertDialog.Builder(vncCanvasActivity)
						.setTitle(vncCanvasActivity.getString(R.string.Instructions))
						.setMessage(vncCanvasActivity.getString(R.string.inst_orientation))
			            .setPositiveButton(android.R.string.ok, new OnClickListener(){
							@Override
							public void onClick(DialogInterface dialog, int which) {
								vncCanvasActivity.setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_LANDSCAPE);
								
						        sensorManager.registerListener(sensorEventListener, sensor,SensorManager.SENSOR_DELAY_UI);
						        registrar_posicion=true;
						        
								
							}
			            }).create().show();
					}
				 return true;
			case BRUJULA:
				encontrado = false;
				sensorManager.unregisterListener(sensorEventListener);
				for (int i=0; i<lista_sensores.size();i++){
					sensor = lista_sensores.get(i);
					Log.d(TAG, "sensor: "+sensor.toString());
					if(sensor.getType()==Sensor.TYPE_MAGNETIC_FIELD){
						encontrado=true;
						break;
						}
				}
				if(encontrado){
					new AlertDialog.Builder(vncCanvasActivity)
		            .setTitle(vncCanvasActivity.getString(R.string.Instructions))
		            .setMessage(vncCanvasActivity.getString(R.string.inst_compass))
		            .setPositiveButton(android.R.string.ok, new OnClickListener(){
						@Override
						public void onClick(DialogInterface dialog, int which) {
							vncCanvasActivity.setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_PORTRAIT);
							sensor = lista_sensores.get(1);//Magnetic
					        sensorManager.registerListener(sensorEventListener, sensor,SensorManager.SENSOR_DELAY_NORMAL);
					        registrar_posicion=true;
						}
		            }).create().show();
				}
		        return true;
			}
			return false;
			}
	  		
	/**
	 * Elimina sensores registrados con el oyente. Se restablece la orientacion automatica segun la posicion del telefono y se notifica al usuario
	 * @author Alberto Poblacion Navarro
	 */
	public void quitar(){
		vncCanvasActivity.setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_SENSOR);                                       
		sensorManager.unregisterListener(sensorEventListener);
		log.log(TAG, "Sensores desactivados");
		notificacion.notificar("Sensores desactivados");
		log.log(TAG, "Orientacion de la pantalla segun la posicion del telefono");
        registrar_posicion=true;
		
	}

	/**
	 * Se libera la instancia para que se cree una nueva la proxima vez. 
	 */
	protected static void finalizar(){
		if(instanciado()){
			instancia.quitar();	
		}
		instancia = null;
	}
	/**
	 * Devuelve true si se ha creado una instancia, false en caso contrario
	 * @return Boleean dependiendo de si ha creado una instancia previa o no.
	 */
	static boolean instanciado(){
		if (instancia == null)
			return false;
		else return true; 
	}
		

}
	
	
	

